# Exercise Fast Courier (textbook, page 206)
# Optimal sizing of a fleet

# DATA
param N;					# Number of time periods (weeks)
set Weeks := 1..N;			# Set of time periods (weeks)
param Demand {Weeks};		# Demand for each week [n. of vehicles]
param Fixed_cost;			# Fixed cost for owning a vehicle [$/week]
param Var_cost;				# Variable cost for operating a vehicle [$/week]
param Rental_cost;			# Cost for renting a vehicle for a week [$/week] 

# VARIABLES
var x integer >= 0;			# N. of vehicles in the fleet
var u {Weeks} >= 0;			# N. of vehicles used in each week
var y {Weeks} >= 0;			# N. of rented vehicles for each week

# CONSTRAINTS
# Demand satisfaction constraint [n. of vehicles]
subject to Demand_satisfaction {t in Weeks}:
  u[t] + y[t] = Demand[t];

# Maximum number of available vehicles [n. of vehicles]
subject to Max_vehicles {t in Weeks}:
  u[t] <= x;

# OBJECTIVE
# Cost minimization: fixed + variable + renting [$/year]
minimize cost: N * Fixed_cost * x
             + sum {t in Weeks} Var_cost * u[t]
			 + sum {t in Weeks} Rental_cost * y[t];

#############################
data;

param N := 52;
param Demand :=
 1	12
 2	15
 3	16
 4	17
 5	17
 6	18
 7	20
 8	20
 9	21
10	22
11	24
12	22
13	20
14	18
15	17
16	16
17	14
18	13
19	13
20	14
21	15
22	16
23	17
24	19
25	21
26	22
27	23
28	22
29	24
30	26
31	27
32	28
33	30
34	32
35	32
36	30
37	29
38	28
39	26
40	25
41	25
42	24
43	22
44	22
45	19
46	20
47	18
48	17
49	16
50	16
51	14
52	13;

param Fixed_cost := 350;
param Var_cost := 150;
param Rental_cost := 800;

end;
