# Exercise Boscheim (page 215)
# Min cost flow, two echelons, linear cost, node capacities, single commodity

# DATA
set O;				# Set of origins (production plant)
set T;				# Set of transshipment nodes (warehouses)
set D;				# Set of destinations (markets)
param Cost1 {O,T};	# First echelon unit cost [/unit]
param Cost2 {T,D};	# Second echelon unit cost [/unit]
param Demand {D};	# Demand for each market [unit/year]
param Capacity {T};	# Capacity of each warehouse [unit/year]

# VARIABLES
var x1 {O,T} >=0;	# First echelon flow [unit/year]
var x2 {T,D} >=0;	# Second echelon flow [unit/year]

# CONSTRAINTS
# Flow balance constraints [unit/year]
subject to Flow_balance_T {t in T}:
  sum {o in O} x1[o,t] = sum {d in D} x2[t,d];

# Flow balance at destinations [unit/year]
subject to Flow_balance_D {d in D}:
  sum {t in T} x2[t,d] = Demand[d];

# Capacity constraints on warehouses [unit/year]
subject to Capacity_T {t in T}:
  sum {o in O} x1[o,t] <= Capacity[t];

# OBJECTIVE
# Minimize the total transportation cost [/year]
minimize z: sum {o in O, t in T} Cost1[o,t] * x1[o,t]
          + sum {t in T, d in D} Cost2[t,d] * x2[t,d];

###################################
data;

set O := Rotterdam;
set T := Bristol Middlesborough;
set D := London Birmingham Leeds Edinburgh;

param Cost1:	Bristol		Middlesborough :=
Rotterdam		24.5		26.0;

param Cost2 (tr):	Bristol		Middlesborough :=
London				 9.6		19.5
Birmingham			 7.0		13.3
Leeds				15.2		 5.0
Edinburgh			28.5		11.3;

param Demand :=
London		90000
Birmingham	80000
Leeds		50000
Edinburgh	70000;

param Capacity :=
Bristol			150000
Middlesborough	150000;

end;
