# Exercise Exofruit (page 221)
# Min cost flow, 2-commodities, capacity, multi-period

# DATA
param nT;				# Number of time periods
param nP;				# Number of products
set Products := 1..nP;	# Set of Products
set Periods := 1..nT;	# Set of Periods
param Demand {Products,Periods};	# Demand [ton/period]
param Available {Products,Periods};	# Max. quantity [ton/period]
param Price {Products,Periods};		# Price [/ton]
param tr_cost;						# Transportation unit cost [/ton]
param st_cost;						# Storage unit cost [/ton period]
param st_cap;						# Capacity [ton]

# VARIABLES
var x1 {p in Products,t in Periods} >=0, <=Available[p,t];	# Incoming flow [ton]
var s {Products,Periods} >=0;								# Stock at the end of each period [ton]

# CONSTRAINTS
subject to Demand_satisfaction {p in Products, t in Periods: (t>1)}:
  x1[p,t] + s[p,t-1] = Demand[p,t] + s[p,t];
subject to Demand_satisfaction1 {p in Products}:
  x1[p,1] + s[p,nT] = Demand[p,1] + s[p,1];

# Capacity constraints
subject to Capacity {t in Periods}:
  sum {p in Products} s[p,t] <= st_cap;

# OBJECTIVE
# Minimize cost = purchase + transportation + storage
minimize z: sum {p in Products, t in Periods} Price[p,t] * x1[p,t]
          + sum {p in Products, t in Periods} tr_cost * (x1[p,t] + Demand[p,t])
          + sum {p in Products, t in Periods} st_cost * s[p,t];

############################
data;

param nT := 2;
param nP := 2;

param Demand:	1		2 :=
1				18000	18000
2				12000	14000;

param Available:	1		2 :=
1				26000	20000
2				14000	13000;

param Price:	1		2 :=
1				500		700
2				600		400;

param tr_cost := 100;	# /ton
param st_cost := 100;	# /ton period
param st_cap  := 8000;	# ton

end;
