# Exercise FHL (textbook, page 229)
# Fixed charge network flow

# DATA
set Cities;
param d_refr {Cities,Cities};	# pallets
param d_norm {Cities,Cities};	# pallets
param km_cost_refr;				# /km
param km_cost_norm;				# /km
param unit_cost_carr;			# /(km*pallet)
param cap_refr;					# pallets
param cap_norm;					# pallets
param dist {Cities,Cities};		# km

# VARIABLES
var x_refr {Cities,Cities,Cities,Cities} >=0; # (arc tail, arc head,origin,destination)
var x_norm {Cities,Cities,Cities,Cities} >=0;
var x_carr {Cities,Cities} >=0;
var y_refr {Cities,Cities} integer >=0;
var y_norm {Cities,Cities} integer >=0;

# CONSTRAINTS
# Flow balance constraints in transshipment nodes
subject to Flow_balance_refr {o in Cities, d in Cities, j in Cities: (j<>o) and (j<>d)}:
  sum {i in Cities} x_refr[i,j,o,d] = sum {k in Cities} x_refr[j,k,o,d];
subject to Flow_balance_norm {o in Cities, d in Cities, j in Cities: (j<>o) and (j<>d)}:
  sum {i in Cities} x_norm[i,j,o,d] = sum {k in Cities} x_norm[j,k,o,d];
# Flow balance at origins
subject to Flow_balance_refr_o {o in Cities, d in Cities}:
  sum {k in Cities: (k<>o)} x_refr[o,k,o,d] = d_refr[o,d];
subject to Flow_balance_norm_o {o in Cities, d in Cities}:
  sum {k in Cities: (k<>o)} x_norm[o,k,o,d] + x_carr[o,d] = d_norm[o,d];
# Flow balance at destinations
subject to Flow_balance_refr_d {o in Cities, d in Cities}:
  sum {i in Cities: (i<>d)} x_refr[i,d,o,d] = d_refr[o,d];
subject to Flow_balance_norm_d {o in Cities, d in Cities}:
  sum {i in Cities: (i<>d)} x_norm[i,d,o,d] + x_carr[o,d] = d_norm[o,d];
# Capacity of each line
subject to Line_capacity_refr {i in Cities, j in Cities: (i<>j)}:
  sum {o in Cities, d in Cities: (o<>d)} x_refr[i,j,o,d] <= cap_refr * y_refr[i,j];
subject to Line_capacity_norm {i in Cities, j in Cities: (i<>j)}:
  sum {o in Cities, d in Cities: (o<>d)} x_norm[i,j,o,d] <= cap_norm * y_norm[i,j];

# OBJECTIVE
minimize z: 
  sum {i in Cities, j in Cities: (i<>j)} km_cost_refr * dist[i,j] * y_refr[i,j]
+ sum {i in Cities, j in Cities: (i<>j)} km_cost_norm * dist[i,j] * y_norm[i,j]
+ sum {o in Cities, d in Cities: (o<>d)} unit_cost_carr * dist[o,d] * x_carr[o,d];

#######################
data;

set Cities := BO GE MI PD;

param d_refr: 	BO	GE	MI	PD :=
BO				0	3	8	2
GE				0	0	1	2
MI				4	2	0	1
PD				3	1	1	0;

param d_norm: 	BO	GE	MI	PD :=
BO				0	3	4	2
GE				1	0	1	0
MI				6	2	0	2
PD				1	1	1	0;

param km_cost_refr := 0.4;
param km_cost_norm := 0.5;
param unit_cost_carr := 0.1;
param cap_refr := 12;
param cap_norm := 18;

param dist: BO	GE	MI	PD :=
BO			  0	225	115	292
GE			225	  0	226	166
MI			115	226	  0	362
PD			292	166	362	  0;

end;
